X-Git-Url: https://git.madduck.net/etc/awesome.git/blobdiff_plain/1eb2829cace5dbb35b7b9d5eb0df838296f21589..fc5974a9c2838efa8dedbd350d9623e193ab71a3:/widgets/bat.lua?ds=sidebyside

diff --git a/widgets/bat.lua b/widgets/bat.lua
index 54959eb..8b62790 100644
--- a/widgets/bat.lua
+++ b/widgets/bat.lua
@@ -7,18 +7,17 @@
 												                        
 --]]
 
-local newtimer     = require("lain.helpers").newtimer
 local first_line   = require("lain.helpers").first_line
-
+local make_widget  = require("lain.helpers").make_widget_textbox
+local newtimer     = require("lain.helpers").newtimer
 local naughty      = require("naughty")
 local wibox        = require("wibox")
-
 local math         = { abs    = math.abs,
                        floor  = math.floor,
                        log10  = math.log10,
                        min    = math.min }
 local string       = { format = string.format }
-
+local ipairs       = ipairs
 local type         = type
 local tonumber     = tonumber
 local setmetatable = setmetatable
@@ -27,7 +26,7 @@ local setmetatable = setmetatable
 -- lain.widgets.bat
 
 local function worker(args)
-    local bat       = {}
+    local bat       = make_widget()
     local args      = args or {}
     local timeout   = args.timeout or 30
     local batteries = args.batteries or (args.battery and {args.battery}) or {"BAT0"}
@@ -35,8 +34,6 @@ local function worker(args)
     local notify    = args.notify or "on"
     local settings  = args.settings or function() end
 
-    bat.widget = wibox.widget.textbox('')
-
     bat_notification_low_preset = {
         title   = "Battery low",
         text    = "Plug the cable!",
@@ -62,12 +59,13 @@ local function worker(args)
     }
 
     bat_now.n_status = {}
+    bat_now.n_perc   = {}
     for i = 1, #batteries do
         bat_now.n_status[i] = "N/A"
         bat_now.n_perc[i] = 0
     end
 
-    function update()
+    function bat.update()
         local sum_rate_current = 0
         local sum_rate_voltage = 0
         local sum_rate_power   = 0
@@ -98,7 +96,7 @@ local function worker(args)
                                           math.floor((energy_now / energy_full) * 100)
 
                 bat_now.n_status[i] = first_line(bstr .. "/status") or "N/A"
-                bat_now.perc[i]     = energy_percentage or bat_now.perc[i]
+                bat_now.n_perc[i]   = energy_percentage or bat_now.n_perc[i]
 
                 sum_rate_current = sum_rate_current + (rate_current or 0)
                 sum_rate_voltage = sum_rate_voltage + (rate_voltage or 0)
@@ -109,25 +107,42 @@ local function worker(args)
             end
         end
 
+        -- When one of the battery is charging, others' status are either
+        -- "Full", "Unknown" or "Charging". When the laptop is not plugged in,
+        -- one or more of the batteries may be full, but only one battery
+        -- discharging suffices to set global status to "Discharging".
         bat_now.status = bat_now.n_status[1]
+        for _,status in ipairs(bat_now.n_status) do
+            if status == "Discharging" or status == "Charging" then
+                bat_now.status = status
+            end
+        end
         bat_now.ac_status = tonumber(first_line(string.format("%s%s/online", pspath, ac))) or "N/A"
 
         if bat_now.status ~= "N/A" then
+            if bat_now.status ~= "Full" and sum_rate_power == 0 and bat_now.ac_status == 1 then
+								bat_now.perc  = math.floor(math.min(100, (sum_energy_now / sum_energy_full) * 100))
+								bat_now.time  = "00:00"
+								bat_now.watt  = 0
+
             -- update {perc,time,watt} iff battery not full and rate > 0
-            if bat_now.status ~= "Full" and (sum_rate_power > 0 or sum_rate_current > 0) then
+            elseif bat_now.status ~= "Full" then
                 local rate_time = 0
-                local div = (sum_rate_power > 0 and sum_rate_power) or sum_rate_current
-
-                if bat_now.status == "Charging" then
-                    rate_time = (sum_energy_full - sum_energy_now) / div
-                else -- Discharging
-                    rate_time = sum_energy_now / div
-                end
-
-                if rate_time < 0.01 then -- check for magnitude discrepancies (#199)
-                    rate_time_magnitude = math.abs(math.floor(math.log10(rate_time)))
-                    rate_time = rate_time * 10^(rate_time_magnitude - 2)
-                end
+                -- Calculate time and watt if rates are greater then 0
+                if (sum_rate_power > 0 or sum_rate_current > 0) then
+                    local div = (sum_rate_power > 0 and sum_rate_power) or sum_rate_current
+
+                    if bat_now.status == "Charging" then
+                        rate_time = (sum_energy_full - sum_energy_now) / div
+                    else -- Discharging
+                        rate_time = sum_energy_now / div
+                    end
+
+                    if 0 < rate_time and rate_time < 0.01 then -- check for magnitude discrepancies (#199)
+                        rate_time_magnitude = math.abs(math.floor(math.log10(rate_time)))
+                        rate_time = rate_time * 10^(rate_time_magnitude - 2)
+                    end
+                 end
 
                 local hours   = math.floor(rate_time)
                 local minutes = math.floor((rate_time - hours) * 60)
@@ -160,8 +175,8 @@ local function worker(args)
         end
     end
 
-    newtimer(battery, timeout, update)
-
+    newtimer("batteries", timeout, bat.update)
+    
     return setmetatable(bat, { __index = bat.widget })
 end